import ziptag from './'
import sigmund from 'sigmund'

import http from 'http'
import { Benchmark } from 'benchmark'
import randomObj from './test/helpers/randomObj'

function sparseFE2 (obj, maxDepth) {
  var seen = []
  var soFar = ''
  function ch (v, depth) {
    if (depth > maxDepth) return
    if (typeof v === 'function' || typeof v === 'undefined') return
    if (typeof v !== 'object' || !v) {
      soFar += v
      return
    }
    if (seen.indexOf(v) !== -1 || depth === maxDepth) return
    seen.push(v)
    soFar += '{'
    Object.keys(v).forEach(function (k, _, __) {
      // pseudo-private values.  skip those.
      if (k.charAt(0) === '_') return
      var to = typeof v[k]
      if (to === 'function' || to === 'undefined') return
      soFar += k + ':'
      ch(v[k], depth + 1)
    })
    soFar += '}'
  }
  ch(obj, 0)
  return soFar
}

// run test against given entity
function runSuite (entity) {
  return new Promise((resolve, reject) => {
    const suite = new Benchmark.Suite()

    suite
      .add('ziptag', () => ziptag(entity))
      .add('sigmund', () => sigmund(entity, 10))
      .add('JSON.stringify', () => JSON.stringify(entity))
      .add('sparseFE2', () => sparseFE2(entity, 10))

      .on('cycle', evt => console.log(String(evt.target)))
      .on('complete', function () {
        const fastest = String(this.filter('fastest').map('name'))
        console.log('  => Fastest: ' + fastest)

        if (fastest === 'ziptag') resolve()
        else reject(new Error(`${fastest} is faster than ziptag now.`))
      })
      .run({ 'async': true })
  })
}

function test (text, entity) {
  console.log('')
  console.log(text)
  console.log('-'.repeat(text.length))
  return runSuite(entity)
}

;(async () => {
  await test('For tiny object:', randomObj(10))
  await test('For giant object:', randomObj(100))
})().catch(e => {
  console.error(e)
  process.exit(-1)
})