'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createEtag;
/**
 * @file src/index.js
 * @license MIT
 * @copyright 2017-present Foko Inc. All rights reserved.
 */

var getPrototypeOf = Object.getPrototypeOf || function legacyGetPrototypeOf(o) {
  /* eslint no-proto: 0 */
  return o.__proto__;
};

/**
 * Creates a dirty JSON signature for quick ETag verification.
 * @param {Object} json stringifiable object
 * @param {Number} [max=10] maximum depth to search
 * @param {Array} seen internally used to manage circular object
 */
function genSig(json) {
  var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 10;
  var seen = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  // protect against circular objects
  if (seen.length >= max || seen.indexOf(json) !== -1 || json === undefined) return '';
  seen.push(json);

  var type = typeof json;

  // quick exit for primitives
  if (type === 'function' || json === null || Number.isNaN(json)) return 'null';
  if (type === 'string') return `'${json.replace(/[^\x21\x23-\x7E\x80-\xFF]+/g, '')}'`;

  // number & boolean
  if (type !== 'object') return json;

  // built-ins
  var proto = getPrototypeOf(json);

  if (proto !== null) {
    if ('toLocaleTimeString' in proto) return +json;
    if ('test' in proto && 'exec' in proto) return json;
  }

  // quick stringify
  var sigg = 'length' in json && 'map' in json ? '[' : '{';
  for (var k in json) {
    sigg += k + ':' + genSig(json[k], max, seen);
  }
  return sigg;
}

/**
 * Generates an ETag for a given entity.
 * @param {any} body entity for which to generate a tag
 * @returns a weak ETag representing the entity
 */
function createEtag(body, depth) {
  return `W/"${genSig(body, depth)}"`;
}